NAME

  Clock - DESQview/X Analog/Digital Clock


DESCRIPTION

  The Clock program displays the time in analog and digital forms.

  The time is continuously updated at a frequency which can be specified by
  the user.

  Press any key or click on any mouse button to switch between a
  conventional 12-hour analog clock, an "ampm" 12-hour digital clock and
  a "military" 24-hour digital clock.

  A stopwatch option is also provided which displays in the 24-hour digital
  format with standard 2-button (Start/Stop and Lap/Reset) functionality.


SYNOPSIS

  clock [options]

  options:

    [{-dp | -display} <display name>]

    [{-ti | -title} <name>]
    [{-g | -geometry} [<width>][x<height>][{+-}<xoffset>[{+-}<yoffset>]]]
    [{-pd | -padding} <pixels>]

    [{-in | -iconname} <icon name>]
    [{-ig | -icongeometry} [<width>][x<height>][{+-}<xoffset>[{+-}<yoffset>]]]
    [{-ic | -iconic}]

    [{-an | -analog} | {-d | -digital} | -ampm]
    [{-ns | -noswitch}]

    [{-sw | -stopwatch}]

    [{-up | -update} <seconds>]
    [{-ch | -chime}]
    [{-of | -offset} [{+-}]<hh[:mm[:ss]]>]

    [{-fn | -font} <font name>]
    [{-sfn | -sfont} -<foundry>[-<family>[-<weight>[-<slant>]]]]

    [{-rv | -reverse}]
    [{-bg | -background} <color>]
    [{-fg | -foreground} <color>]
    [{-hd | -hands}      <color>]
    [{-hl | -highlight}  <color>]
    [{-hr | -hours}      <color>]

    [{-ro | -roman}]
    [{-3 | -mult3}]
    [{-nh | -nohours}]
    [{-nt | -noticks}]

    [-xrm <resource name/class storage key>:<resource value>]
    [-name <resource name retrieval key parent>]

    [-help]


EXAMPLES

  Display three separate clocks with differing titles, hour offsets and window
  positions:

    clock -title Hawaii        -offset -3 -g 160x160+0+0
    clock -title "Los Angeles"            -g 160x160+160+0
    clock -title "New York"    -offset +3 -g 160x160+320+0

  Start three separate iconified clocks with differing titles, hour offsets
  and colors:

    clock -title Hawaii        -offset -3 -iconic -bg green -fg white
    clock -title "Los Angeles"            -iconic -bg yellow
    clock -title "New York"    -offset +3 -iconic -bg LightGray

  Start three separate clocks using the -name option to retrieve differing
  resource values from the default \DVX\APP-DEFA\CLOCK.RES resource database
  file:

    clock -name hawaii
    clock -name losAngeles
    clock -name newYork

  Start a stopwatch clock with a specified title:

    clock -sw -title StopWatch


OPTIONS

  -dp or -display <name>

    Specifies the default host and display number.

  -ti or -title <title>

    Specifies the title for the clock's main and icon windows.

    The default is "Clock".

  -g or -geometry <width>x<height>{+-}<xoffset>{+-}<yoffset>

    Specifies the initial window geometry (dimensions and position) in pixels.

    The default is "164x164+0+0" for analog and "164x41+0+0" for digital clocks.

  -pd or -padding <pixels>

    Specifies the width in pixels of the padding between the window border and
    face of an analog clock.

    The default is 0.

  -in or -iconname <title>

    Specifies the title for the clock's icon window.

    The default is "Clock".

  -ig or -icongeometry
    [<width>][x<height>]
      [{+-}<xoffset>[{+-}<yoffset>]]

    Specifies the icon's window geometry (dimensions and position) in pixels.

    The default is determined by the window manager.

  -ic or -iconic

    Indicates that the clock's window should start up iconified.

    This is not the default.

  -an or -analog

    Indicates that the clock should start up as a conventional 12-hour analog
    clock face with hour numbers (1 - 12), tick marks and hands.

    This is the default.

  -d or -digital

    Indicates that the clock should start up as a 24-hour digital clock.

    The default is an analog clock.

  -ampm

    Indicates that the clock should start up as a 12-hour digital clock that
    also displays "a" when A.M. and "p" when P.M.

    The default is an analog clock.

  -ns or -noswitch

    Indicates that the clock does not switch between 12-hour analog, 12-hour
    digital and 24-hour digital modes when a key is pressed or mouse button is
    clicked.

    This is not the default.

  -sw or -stopwatch

    Indicates that the clock should start up as a 24-hour digital stopwatch
    with the standard 2-button (Start/Stop and Lap/Reset) functionality.
    Press Button 1 or the Space key for Start/Stop.  Press Button 3 or the Esc
    key for Lap/Reset.

    This is not the default.

  -up or -update <seconds>

    Specifies the frequency in seconds at which the clock should update its
    display.  If the clock is obscured and then exposed, it will be updated
    immediately.

    A value of 60 seconds or less will enable a second hand on an analog clock
    and seconds digits on a digital clock.

    The default is 1 second.

  -of or -offset [{+-}]<hh:mm:ss>

    Specifies the plus or minus time offset (in hours[:minutes[:seconds]]) to
    add to the time reported by the system clock.

    The default is 0 (no offset).

  -ch or -chime

    Indicates that the clock should chime once on the half hour and twice on
    the hour.

    This is not the default.

  -fn or -font <font name>

    Specifies the font (scalable or non-scalable) to be used for displaying
    the hour numbers on an analog clock and the digits on a digital clock.

    The default is NULL.

  -sfn or -sfont -<foundry>[-<family>[-<weight>[-<slant>]]]

    Specifies the scalable font's foundry, family, weight and slant to be used
    for displaying the hour numbers on an analog clock and the digits on a
    digital clock.

    The default is "-adobe-times-bold-r".

  -rv or -reverse

    Indicates that the clock colors should display in reverse-video, with the
    background color defaulting to black and the foreground to white.

    This is not the default.

  -bg or -background <color>

    Specifies the background color of the clock.

    If reverse-video is specified, the default is black.  Otherwise, the
    default is white.

  -fg or -foreground <color>

    Specifies the color of the tick marks on an analog clock and the digits
    on a digital clock.

    If reverse-video is specified, the default is white.  Otherwise, the
    default is black.

  -hd or -hands <color>

    Specifies the color of the insides of the hands on an analog clock.

    If reverse-video is specified, the default is white.  Otherwise, the
    default is black.

  -hl or -highlight <color>

    Specifies the color used to highlight the edges of the hands on an analog
    clock.

    If reverse-video is specified, the default is white.  Otherwise, the
    default is black.

  -hr or -hours <color>

    Specifies the color of the hour numbers on the face of an analog clock.

    If reverse-video is specified, the default is white.  Otherwise, the
    default is black.

  -ro or -roman

    Indicates that Roman numerals (I - XII) should be used as the hour numbers
    of an analog clock instead of the normal Arabic numeral digits (1 - 12).
    Note that the Clock program uses the "IIII" convention instead of "IV"
    for the hour number "4".

    This is not the default.

  -3 or -mult3

    Indicates that only the hour numbers which are multiples of 3 (3, 6, 9, 12
    or III, VI, IX, XII) should be displayed on the face of an analog clock.

    This is not the default.

  -nh or -nohours

    Indicates that no hour numbers should be displayed on the face of an analog
    clock.

    This is not the default.

  -nt or -noticks

    Indicates that no tick marks should be displayed on the face of an analog
    clock.

    This is not the default.

  -xrm <resource name/class storage key>:<resource value>

    Stores a fully-specified resource name/class storage key and value pair
    into the application's resource database file(s) for later retrieval.

  -name <resource name retrieval key parent>

    Specifies the parent component of the resource name retrieval key used
    to retrieve values from the application's resource database file(s).

    The default is "clock".

  -help or any unsupported option

    Indicates that a brief summary of the allowed options should be printed on
    the standard error output device.


RESOURCE OPTIONS

  Option            Name                Class              Default Value
  ------            ----                -----              -------------

  -title            title               Title              Clock
  -geometry         geometry            Geometry           164x164
  -padding          padding             Margin             0

  -iconname         iconName            Title              Clock
  -icongeometry     iconGeometry        Geometry           NULL

  -iconic           iconStartup         Boolean            False

  -analog           analog              Boolean            True
  -digital
  -ampm             ampm                Boolean            False
  -noswitch         switchModes         Boolean            True

  -stopwatch        stopWatch           Boolean            False

  -update           update              Interval           1
  -chime            chime               Boolean            False
  -offset           timeOffset          Time               0

  -font             font                Font               NULL
  -sfont            scalableFont        ScalableFont       -adobe-times-bold-r

  -reverse          reverseVideo        ReverseVideo       False
  -foreground       foreground          Foreground         black
  -background       background          Background         white
  -hands            hand                Foreground         black
  -highlight        highlight           Foreground         black
  -hours            hour                Foreground         black

  -roman            romanNumerals       Boolean            False
  -mult3            multiplesOf3        Boolean            False
  -nohours          showHours           Boolean            True
  -noticks          showTicks           Boolean            True


DOS ENVIRONMENT VARIABLES

  DISPLAY=<display name>

  Specifies the default host and display number.

  HOME=<home directory>

  Specifies the "home" directory used to form the path of a resource file that
  overrides the global resources.

  XENVIRONMENT=<resource file>

  Specifies the name of a resource file that overrides the global resources.


RESOURCE SPECIFICATIONS

  \DVX\APP-DEFA\CLOCK.RES resource file

  RESOURCE_MANAGER server property

    %HOME%\XDEFAULT.RES resource file

  %XENVIRONMENT% resource file

    %HOME%\<host>\XDEFAULT.RES resource file

  command-line options


RESOURCE NAME AND CLASS KEY PARENTS

  Name Key Parent:   clock
  Class Key Parent:  Clock


SAMPLE RESOURCE FILE (\DVX\APP-DEFA\CLOCK.RES)

  # class-level resources set to application's default values.

  Clock*Title:              Clock

  Clock*Foreground:         black
  Clock*Background:         white

  Clock.geometry:           164x164
  Clock.padding:            0

  Clock.iconStartup:        False

  Clock.analog:             True
  Clock.ampm:               False
  Clock.switchModes:        True

  Clock.stopWatch:          False

  Clock.update:             1
  Clock.chime:              False
  Clock.timeOffset:         0

  Clock.romanNumerals:      False
  Clock.multiplesOf3:       False
  Clock.showHours:          True
  Clock.showTicks:          True

  # "Hawaii" clock resources invoked by "clock -name hawaii"

  hawaii*Title:             Hawaii
  hawaii.timeOffset:        -3
  hawaii.geometry:          160x160
  hawaii*Background:        green
  hawaii*Foreground:        white

  # "Los Angeles" clock resources invoked by "clock -name losAngeles"

  losAngeles*Title:         Los Angeles
  losAngeles.timeOffset:    0
  losAngeles.geometry:      160x160+160+0
  losAngeles*Background:    yellow

  # "New York" clock resources invoked by "clock -name newYork"

  newYork*Title:            New York
  newYork.timeOffset:       +3
  newYork.geometry:         160x160+320+0
  newYork*Background:       LightGray


COPYRIGHT

  Copyright (c) 1987-89 Digital Equipment Corporation and
    the Massachusetts Institute of Technology.
  Copyright (c) 1992 Quarterdeck Office Systems, Inc.

